(****************************************************************************
*
*                       MegaGraph Graphics Library
*
*               Copyright (C) 1991-1997 SciTech Software, Inc.
*                            All rights reserved.
*
*  ======================================================================
*       This library is free software; you can use it and/or
*       modify it under the terms of the SciTech MGL Software License.
*
*       This library is distributed in the hope that it will be useful,
*       but WITHOUT ANY WARRANTY; without even the implied warranty of
*       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*       SciTech MGL Software License for more details.
*  ======================================================================
*
* Filename:     $Workfile:   mglgm.pas  $
* Version:      $Revision:   1.1  $
*
* Language:     TMT Pascal Multitarget 3
* Environment:  Win32
* C Headers:    gm\gm.h
*
* Description:  Game Framework for SciTech MGL. This library sprovides the
*               common functions that every game written with the MGL
*               will require, and automatically handles all of the 'polish'
*               functionality required of commercial games for the DOS
*               and Windows environments. This library provides support for
*               issues such as the following:
*
*               . Handling alt-tab'ing to GDI mode and back to fullscreen
*               . Static system color management in 8bpp modes
*               . Palette management for 8bpp modes windowed and fullscreen
*               . Switching between fullscreen and windowed modes on the fly
*               . Handling of system menu, maximise and minimise buttons
*               . System buffering in a window or with one hardware page
*               . Hardware multi-buffering for smooth animation fullscreen
*               . 1x2 and 2x2 stretching for both fullscreen and a window
*               . Event handling for mouse and keyboard
*               . Application activation notification
*               . Dirty rectangle management for system buffering
*               . OpenGL rendering support both windowed and fullscreen
*
* $Date:  05 Oct 1997 $ $Author of original C/C++ code:  KendallB  $
* $Date:  10 Apr 1998 $ $Author of TMT Pascal version :  Vadim Bodrov  $
*
****************************************************************************)

unit MGLGm;

interface

uses
    Windows,MGLTypes,MGLFx;

{$OA-}
{$W-}

{-- Constants --------------------------------------------------------------}

const
    GM_STRETCH_1x1      = 0;
    GM_STRETCH_1x2      = 1;
    GM_STRETCH_2x2      = 2;

    GM_MODE_8BPP        = $01;
    GM_MODE_16BPP       = $02;
    GM_MODE_24BPP       = $04;
    GM_MODE_32BPP       = $08;
    GM_ONLY_2D_ACCEL    = $10;
    GM_ONLY_3D_ACCEL    = $20;
    GM_ALLOW_WINDOWED   = $40;
    GM_MODE_ALLBPP      = $0F;

    kbGrayEnter         = $E00D;
    kbGrayMinus         = $4A2D;
    kbGrayPlus          = $4E2B;
    kbGrayTimes         = $372A;
    kbGrayDivide        = $36E0;
    kbF1                = $3B00;
    kbF2                = $3C00;
    kbF3                = $3D00;
    kbF4                = $3E00;
    kbF5                = $3F00;
    kbF6                = $4000;
    kbF7                = $4100;
    kbF8                = $4200;
    kbF9                = $4300;
    kbF10               = $4400;
    kbF11               = $8500;
    kbF12               = $8600;
    kbLeft              = $4B00;
    kbRight             = $4D00;
    kbUp                = $4800;
    kbDown              = $5000;
    kbIns               = $5200;
    kbDel               = $5300;
    kbHome              = $4700;
    kbEnd               = $4F00;
    kbPgUp              = $4900;
    kbPgDn              = $5100;

{-- Structures -------------------------------------------------------------}

type
    PGM_driverOptions   = ^GM_driverOptions;
    GM_driverOptions    = record
        useWinDirect        :longbool;
        useDirectDraw       :longbool;
        useVGA              :longbool;
        useVGAX             :longbool;
        useVBE              :longbool;
        useLinear           :longbool;
        useVBEAF            :longbool;
        useFullscreenDIB    :longbool;
        useHWOpenGL         :longbool;
        openGLType          :int;
        modeFlags           :int;
    end;

    PGM_modeInfo    = ^GM_modeInfo;
    GM_modeInfo     = record
        mode                :int;
        xRes                :int;
        yRes                :int;
        bitsPerPixel        :int;
        pages               :int;
        flags               :ulong;
        driverName          :array [0..12] of char;
        stretch             :int;
        windowedStretch     :int;
    end;

    PGMDC           = ^GMDC;
    GMDC            = record
        dc                  :PMGLDC;
        dispdc              :PMGLDC;
        memdc               :PMGLDC;
        driver              :int;
        numModes            :int;
        numFullscreenModes  :int;
        modeList            :array[0..grMAXMODE] of GM_modeInfo;
        mainWindow          :HWND;
    end;

{-- Type definitions for Game Framework callback functions -----------------}

    GM_drawFunc             = ^procedure conv arg_cdecl;
    GM_keyDownFunc          = ^procedure conv arg_cdecl (evt: pevent_t);
    GM_keyRepeatFunc        = ^procedure conv arg_cdecl (evt: pevent_t);
    GM_keyUpFunc            = ^procedure conv arg_cdecl (evt: pevent_t);
    GM_mouseDownFunc        = ^procedure conv arg_cdecl (evt: pevent_t);
    GM_mouseUpFunc          = ^procedure conv arg_cdecl (evt: pevent_t);
    GM_mouseMoveFunc        = ^procedure conv arg_cdecl (evt: pevent_t);
    GM_eventFunc            = ^procedure conv arg_cdecl (evt: pevent_t);
    GM_modeSwitchFunc       = ^procedure conv arg_cdecl (var mode: GM_modeInfo;windowed: LongBool);
    GM_preModeSwitchFunc    = ^function conv arg_cdecl (var mode: GM_modeInfo;windowed: LongBool): Boolean;
    GM_gameFunc             = ^procedure conv arg_cdecl;
    GM_exitFunc             = ^procedure conv arg_cdecl;
    GM_activateFunc         = ^procedure conv arg_cdecl (active: LongBool);
    GM_modeFilterFunc       = ^function conv arg_cdecl (xres,yres,bits: int; flags: ulong): Boolean;

{== Function prototypes ====================================================}

procedure   GM_initSysPalNoStatic conv arg_cdecl (flag: LongBool);
procedure   GM_initPath conv arg_cdecl (MGLPath: PChar);
procedure   GM_initWindowPos conv arg_cdecl (x,y: int);
procedure   GM_setModeFilterFunc conv arg_cdecl (filter: GM_modeFilterFunc);
procedure   GM_setDriverOptions conv arg_cdecl (var opt: GM_driverOptions);
function    GM_init conv arg_cdecl (windowTitle: PChar): PGMDC;

procedure   GM_setDrawFunc conv arg_cdecl (func: GM_drawFunc);
procedure   GM_setKeyDownFunc conv arg_cdecl (func: GM_keyDownFunc);
procedure   GM_setKeyRepeatFunc conv arg_cdecl (func: GM_keyRepeatFunc);
procedure   GM_setKeyUpFunc conv arg_cdecl (func: GM_keyUpFunc);
procedure   GM_setMouseDownFunc conv arg_cdecl (func: GM_mouseDownFunc);
procedure   GM_setMouseUpFunc conv arg_cdecl (func: GM_mouseUpFunc);
procedure   GM_setMouseMoveFunc conv arg_cdecl (func: GM_mouseMoveFunc);
procedure   GM_setEventFunc conv arg_cdecl (func: GM_eventFunc);
procedure   GM_setGameLogicFunc conv arg_cdecl (func: GM_gameFunc);
procedure   GM_setExitFunc conv arg_cdecl (func: GM_exitFunc);
procedure   GM_setAppActivate conv arg_cdecl (func: GM_activateFunc);
procedure   GM_setPreModeSwitchFunc conv arg_cdecl (func: GM_preModeSwitchFunc);
procedure   GM_setModeSwitchFunc conv arg_cdecl (func: GM_modeSwitchFunc);
procedure   GM_setSuspendAppCallback conv arg_cdecl (saveState: MGL_suspend_cb_t);
procedure   GM_registerEventProc conv arg_cdecl (winproc: TFarProc);
procedure   GM_registerMainWindow conv arg_cdecl (hwndMain: HWND);

function    GM_chooseMode conv arg_cdecl (var mode: GM_modeInfo; var startWindowed: LongBool): Boolean;
function    GM_findMode conv arg_cdecl (var mode: GM_modeInfo; xRes: int; yRes: int; bits: int): Boolean;
function    GM_setMode conv arg_cdecl (var info: GM_modeInfo; windowed: LongBool; pages: int; forceSysMem: LongBool): Boolean;

function    GM_startOpenGL conv arg_cdecl (flags: int): Boolean;
procedure   GM_swapBuffers conv arg_cdecl (waitVRT: int);
procedure   GM_swapDirtyBuffers conv arg_cdecl (dirty: pregion_t; waitVRT: int);
procedure   GM_setPalette conv arg_cdecl (pal: ppalette_t; numColors: int; startIndex: int);
procedure   GM_realizePalette conv arg_cdecl (numColors: int; startIndex: int; waitVRT: LongBool);
procedure   GM_mainLoop conv arg_cdecl ;
procedure   GM_processEvents conv arg_cdecl ;
procedure   GM_processEventsWin conv arg_cdecl ;
procedure   GM_cleanUp conv arg_cdecl ;
procedure   GM_exit conv arg_cdecl ;
function    GM_getHaveWin31 conv arg_cdecl : Boolean;
function    GM_getHaveWin95 conv arg_cdecl : Boolean;
function    GM_getHaveWinNT conv arg_cdecl : Boolean;
function    GM_getDoDraw conv arg_cdecl : Boolean;
function    GM_getExitMainLoop conv arg_cdecl : Boolean;

{===========================================================================}

implementation

const
    DLLName = 'MGLGM.DLL';

procedure   GM_initSysPalNoStatic conv arg_cdecl (flag: LongBool);  external DLLName name '_GM_initSysPalNoStatic';
procedure   GM_initPath conv arg_cdecl (MGLPath: PChar);  external DLLName name '_GM_initPath';
procedure   GM_initWindowPos conv arg_cdecl (x,y: int);  external DLLName name '_GM_initWindowPos';
procedure   GM_setModeFilterFunc conv arg_cdecl (filter: GM_modeFilterFunc);  external DLLName name '_GM_setModeFilterFunc';
procedure   GM_setDriverOptions conv arg_cdecl (var opt: GM_driverOptions);  external DLLName name '_GM_setDriverOptions';
function    GM_init conv arg_cdecl (windowTitle: PChar): PGMDC;  external DLLName name '_GM_init';

procedure   GM_setDrawFunc conv arg_cdecl (func: GM_drawFunc);  external DLLName name '_GM_setDrawFunc';
procedure   GM_setKeyDownFunc conv arg_cdecl (func: GM_keyDownFunc);  external DLLName name '_GM_setKeyDownFunc';
procedure   GM_setKeyRepeatFunc conv arg_cdecl (func: GM_keyRepeatFunc);  external DLLName name '_GM_setKeyRepeatFunc';
procedure   GM_setKeyUpFunc conv arg_cdecl (func: GM_keyUpFunc);  external DLLName name '_GM_setKeyUpFunc';
procedure   GM_setMouseDownFunc conv arg_cdecl (func: GM_mouseDownFunc);  external DLLName name '_GM_setMouseDownFunc';
procedure   GM_setMouseUpFunc conv arg_cdecl (func: GM_mouseUpFunc);  external DLLName name '_GM_setMouseUpFunc';
procedure   GM_setMouseMoveFunc conv arg_cdecl (func: GM_mouseMoveFunc);  external DLLName name '_GM_setMouseMoveFunc';
procedure   GM_setEventFunc conv arg_cdecl (func: GM_eventFunc);  external DLLName name '_GM_setEventFunc';
procedure   GM_setGameLogicFunc conv arg_cdecl (func: GM_gameFunc);  external DLLName name '_GM_setGameLogicFunc';
procedure   GM_setExitFunc conv arg_cdecl (func: GM_exitFunc);  external DLLName name '_GM_setExitFunc';
procedure   GM_setAppActivate conv arg_cdecl (func: GM_activateFunc);  external DLLName name '_GM_setAppActivate';
procedure   GM_setPreModeSwitchFunc conv arg_cdecl (func: GM_preModeSwitchFunc);  external DLLName name '_GM_setPreModeSwitchFunc';
procedure   GM_setModeSwitchFunc conv arg_cdecl (func: GM_modeSwitchFunc);  external DLLName name '_GM_setModeSwitchFunc';
procedure   GM_setSuspendAppCallback conv arg_cdecl (saveState: MGL_suspend_cb_t); external DLLName name '_GM_setSuspendAppCallback';
procedure   GM_registerEventProc conv arg_cdecl (winproc: TFarProc);  external DLLName name '_GM_registerEventProc';
procedure   GM_registerMainWindow conv arg_cdecl (hwndMain: HWND);  external DLLName name '_GM_registerMainWindow';

function    GM_chooseMode conv arg_cdecl (var mode: GM_modeInfo; var startWindowed: LongBool): Boolean;  external DLLName name '_GM_chooseMode';
function    GM_findMode conv arg_cdecl (var mode: GM_modeInfo; xRes: int; yRes: int; bits: int): Boolean;  external DLLName name '_GM_findMode';
function    GM_setMode conv arg_cdecl (var info: GM_modeInfo; windowed: LongBool; pages: int; forceSysMem: LongBool): Boolean;  external DLLName name '_GM_setMode';

function    GM_startOpenGL conv arg_cdecl (flags: int): Boolean;  external DLLName name '_GM_startOpenGL';
procedure   GM_swapBuffers conv arg_cdecl (waitVRT: int);  external DLLName name '_GM_swapBuffers';
procedure   GM_swapDirtyBuffers conv arg_cdecl (dirty: pregion_t; waitVRT: int);  external DLLName name '_GM_swapDirtyBuffers';
procedure   GM_setPalette conv arg_cdecl (pal: ppalette_t; numColors: int; startIndex: int);  external DLLName name '_GM_setPalette';
procedure   GM_realizePalette conv arg_cdecl (numColors: int; startIndex: int; waitVRT: LongBool);  external DLLName name '_GM_realizePalette';
procedure   GM_mainLoop conv arg_cdecl ;  external DLLName name '_GM_mainLoop';
procedure   GM_processEvents conv arg_cdecl ;  external DLLName name '_GM_processEvents';
procedure   GM_processEventsWin conv arg_cdecl ;  external DLLName name '_GM_processEventsWin';
procedure   GM_cleanUp conv arg_cdecl ;  external DLLName name '_GM_cleanUp';
procedure   GM_exit conv arg_cdecl ;  external DLLName name '_GM_exit';
function    GM_getHaveWin31 conv arg_cdecl : Boolean;  external DLLName name '_GM_getHaveWin31';
function    GM_getHaveWin95 conv arg_cdecl : Boolean;  external DLLName name '_GM_getHaveWin95';
function    GM_getHaveWinNT conv arg_cdecl : Boolean;  external DLLName name '_GM_getHaveWinNT';
function    GM_getDoDraw conv arg_cdecl : Boolean;  external DLLName name '_GM_getDoDraw';
function    GM_getExitMainLoop conv arg_cdecl : Boolean;  external DLLName name '_GM_getExitMainLoop';

end.
